#include <iostream>
#include <vector>

using namespace std;

struct planet{
  long long mass;
  long long x_pos;
  long long y_pos;
  long long z_pos;
  long long x_vel;
  long long y_vel;
  long long z_vel;
};

long long gcd(long long m, long long n){
  while(n != 0){
    long long r = m%n;
    m = n;
    n = r;
  }
  return m;
}

long long inverse_mod(long long a,long long b){
  long long x,y,u,v;
  x = 0; 
  y = 1;
  u = 1; 
  v = 0;
  while(a != 0){
    long long q = b/a;
    long long r = b% a;
    long long m = x-u*q;
    long long n = y-v*q;
    b = a;
    a = r;
    x = u;
    y = v;
    u = m;
    v = n;
  }
  return x;
}
  
  

bool Get_Collision_In_Dimension(long long a, long long b, long long& mod, long long& time){
  long long divisor = gcd(a,mod);
  if(b%divisor != 0)
    return false;
  else{
    a=a/divisor;
    mod = mod / divisor;
    long long inverse = inverse_mod(a,mod);
    b = (b/divisor*inverse);
    // if (b < 0)
    //    b = b + mod;
    time = b;
    return true;

  }
}

// how many seconds till the next collision, or -1 if it'll never happen
long long Get_Next_Collision(const vector<planet>& system, long long nx, long long ny, long long nz){
 
  long long soonest = nx*ny*nz;
  long long old_max = nx*ny*nz;
  
  for(long long i = 0; i < system.size(); i++)
    for(long long j = i+1; j < system.size(); j++){
      long long cur = 0;
    long long nnx = nx;
      long long nny = ny;
      long long nnz = nz;
      // when do planets i and j collide?
      long long a1 = system[i].x_vel - system[j].x_vel;
      long long b1 = system[j].x_pos - system[i].x_pos;
      if(a1 < 0){
	a1 = a1 * -1;
	b1 = b1 * -1;
      }
      long long time1;
      if(!Get_Collision_In_Dimension(a1,b1,nnx, time1))
	cur = -1;
      //  if(time1 == 0)
      //	time1 = 1;
      long long m = nnx;
      long long n = time1;
      n=n%m;
      if(n < 0)
	n = n + m;

      long long a2 = system[i].y_vel - system[j].y_vel;

      long long b2 = system[j].y_pos - system[i].y_pos;
      if(a2 < 0){
	a2 = a2 * -1;
	b2 = b2 * -1;
      }
      b2=b2-a2*n;
      a2=a2*m;
    
      long long time2;
      if(cur != -1 && !Get_Collision_In_Dimension(a2,b2,nny,time2))
	cur = -1;
      //   if(time2 == 0)
      //	time2 = 1;
      n=n+time2*m;
      m=m*nny;
      n = n % m;
      if(n < 0)
	n = n + m;

      long long a3 = system[i].z_vel - system[j].z_vel;
      long long b3 = system[j].z_pos - system[i].z_pos;

      if(a3 < 0){
	a3 = a3 * -1;
	b3 = b3 * -1;
      }
      b3=b3-a3*n;
      a3=a3*m;
      long long time3;
      if(cur != -1 && !Get_Collision_In_Dimension(a3,b3, nnz, time3))
	cur = -1;
      // if(time3 == 0)
      //	time3 = 1;
     
      if(cur != -1){
	cur=n+time3*m;
	m=m*nnz;
	cur = cur%m;
	
	if(cur < 0)
	  cur = cur+m;
	//	cur = (time1*time2*time3)%(nx*ny*nz);
	if(cur < soonest)
	  soonest = cur;
      }
    }
  if(soonest == old_max)
    return -1;
  else
    return soonest;
}
	 
void Move(vector<planet>&  system, long long time, long long nx, long long ny, long long nz){
  //  cout << "Moving " << time << " seconds" << endl;
  for(long long i = 0; i < system.size(); i++){
    long long temp = system[i].x_vel*time;
    system[i].x_pos += temp;
    system[i].x_pos = system[i].x_pos % nx; 
    if(system[i].x_pos < 0)
      system[i].x_pos += nx;
    system[i].y_pos = (system[i].y_pos + system[i].y_vel*time) % ny;
    if(system[i].y_pos < 0)
      system[i].y_pos += ny;
    system[i].z_pos = (system[i].z_pos + system[i].z_vel*time) % nz;
    if(system[i].z_pos < 0)
      system[i].z_pos += nz;
  }
 
}

void Collide(vector<planet>& system, long long nx, long long ny, long long nz){
  
  vector<long long> collisions;

  for(long long i = 0; i < system.size(); i++){
    for(long long j = i+1; j < system.size(); j++){
      if(system[i].x_pos == system[j].x_pos &&
	 system[i].y_pos == system[j].y_pos &&
	 system[i].z_pos == system[j].z_pos &&
	 system[j].mass > 0)
	collisions.push_back(j);
    }
    if(collisions.size() > 0){
	for(long long j = 0; j < collisions.size(); j++){
	  system[i].mass += system[collisions[j]].mass;
	  system[i].x_pos += system[collisions[j]].x_pos;
	  system[i].y_pos += system[collisions[j]].y_pos;
	  system[i].z_pos += system[collisions[j]].z_pos;
	  system[i].x_vel += system[collisions[j]].x_vel;
	  system[i].y_vel += system[collisions[j]].y_vel;
	  system[i].z_vel += system[collisions[j]].z_vel;
	  system[collisions[j]].mass = -1;
	}
	long long n = collisions.size() +1;

	system[i].x_pos /= n;
	system[i].y_pos /= n;
	system[i].z_pos /= n;
	system[i].x_vel /= n;
	system[i].y_vel /= n;
	system[i].z_vel /= n;
    }
    collisions.resize(0);
  }   
  vector<planet> temp;
  for(long long i = 0; i < system.size(); i++)
    if(system[i].mass > 0)
      temp.push_back(system[i]);
  system = temp;
}
  

void Swap(planet& x, planet& y){
  planet temp = x;
  x = y;
  y = temp;
}

bool Less_Than(const planet& a, const planet& b){
  if(a.mass < b.mass)
    return true;
  if(a.mass == b.mass){
    if(a.x_pos > b.x_pos)
      return true;
    if(a.x_pos == b.x_pos){
      if(a.y_pos > b.y_pos)
	return true;
      if(a.y_pos == b.y_pos){
	if(a.z_pos > b.z_pos)
	  return true;
      }
    }
  }
  return false;
}   
void Sort(vector<planet>& system){
  for(long long i = 0; i < system.size(); i++)
    for(long long j = 0; j < system.size() -1; j++)
      if(Less_Than(system[j], system[j+1]))
	Swap(system[j], system[j+1]);
}

int main(){
  long long n;
  cin >> n;
  long long cur_case = 1;
    long long n_x, n_y, n_z;
    cin >> n_x >> n_y >> n_z;
    vector<planet> system(n);
    for(long long i = 0; i < n; i++){
      cin >> system[i].mass;
      cin >> system[i].x_pos;
      cin >> system[i].y_pos;
      cin >> system[i].z_pos;
      cin >> system[i].x_vel;
      cin >> system[i].y_vel;
      cin >> system[i].z_vel;
    }
    

      
    long long t = Get_Next_Collision(system, n_x, n_y, n_z);
    while(t > 0){

      Move(system, t, n_x, n_y, n_z);
      Collide(system, n_x, n_y, n_z);
      t = Get_Next_Collision(system, n_x, n_y, n_z);
    }
    Sort(system);
//    cout << "Case " << cur_case << ": " << system.size() << endl;
    cout << system.size() << endl;
    for(long long i = 0; i < system.size(); i++){
      cout << "P" << i << ": " << 
	system[i].mass << " " <<
	system[i].x_pos << " " <<
	system[i].y_pos << " " <<
	system[i].z_pos << " " <<
	system[i].x_vel << " " <<
	system[i].y_vel << " " <<
	system[i].z_vel << endl;
    }
  return 0;
}

	
    
